@@ -1,15 +1,15 @@
 """
 Simple command line tool to access a MVM ESP32 via the serial port
-There are three available commands
 
-For help on usage type: python mvm_control.py -h
+For help on usage type:
+python mvm_control.py -h
 
-For help on command usage type: python mvm_contro.py <get/set/log> -h
+For help on command usage type:
+python mvm_control.py [get/set/log/clog/load/save] -h
 
 Note: The log command can either output to a log file or stdout
 
 The log format is as follows:
-
 {
     "settings": { ... },
     "data": [
@@ -28,9 +28,25 @@ the time of this writing)
 
 Each 'data' entry contains the contents of the 'get all' command, and and an
 extra parameter 'time' that is a unix timestamp of when it was received.
+
+The compact log format is as follows:
+{
+    "settings": { ... },
+    "format": ['field1','field2', ..., 'fieldN']
+    "data": [
+        [ ... ],
+        [ ... ]
+    ]
+}
+
+Where 'format' is a list of the data array field names
+
+Where 'data' is now an array of arrays, with the inner array consisting of the
+raw values read out
 """
 
-__version__ = '1.2.2'
+
+__version__ = '1.2.4'
 
 import serial
 import time
@@ -82,10 +98,12 @@ terminate = False
 
 
 def get_available_serial_ports():
+    """Gets the available serial ports"""
     return [comport.device for comport in list_ports.comports()]
 
 
 def signal_handling(signum, frame):
+    """Capture signal and tells log loop to terminate gracefully"""
     global terminate
     terminate = True
 
@@ -94,6 +112,7 @@ signal.signal(signal.SIGINT, signal_handling)
 
 
 def set_log_settings(log_file, verbose=False):
+    """Sends the log settings to the device"""
     try:
         prev_log = json.load(log_file)
     except Exception as e:
@@ -111,6 +130,7 @@ def set_log_settings(log_file, verbose=False):
 
 
 def get_log_settings(settings_to_store):
+    """Gets the log settings from the device"""
     settings_resp = {}
     for setting in settings_to_store:
         resp = get_mvm_param(ser, setting)
@@ -151,49 +171,60 @@ def _parse_response(ser):
 
 def get_mvm_param(ser, param):
     """Request the ESP32 get the value for a given parameter and transmit it"""
-    try:
-        request = 'get ' + str(param) + '\r\n'
-        ser.write(request.encode('utf-8'))
-        response = _parse_response(ser)
-    except Exception as e:
-        print("Error communicating with device")
-        print(e)
-        return False
+    num_tries = 0
+    while(num_tries < 3):
+        try:
+            request = 'get ' + str(param) + '\r\n'
+            ser.write(request.encode('utf-8'))
+            response = _parse_response(ser)
+        except Exception as e:
+            print("Error communicating with device")
+            print(e)
+            return False
 
-    if(response is not False):
-        if(response == 'no_data'):
-            print("Invalid or unknown command")
-            return False
-        elif(response[0:5] == 'ERROR'):
-            print(response)
-            return False
-        else:
-            return response
+        if(response is not False):
+            if(response == 'no_data'):
+                print("Invalid or unknown command")
+                return False
+            elif(response[0:5] == 'ERROR'):
+                if verbose:
+                    print("Command Error, Retrying...")
+            else:
+                return response
+        num_tries = num_tries + 1
+
+    print(response)
+    return False
 
 
 def set_mvm_param(ser, param, value):
     """Request the ESP32 set a parameter to a given value"""
-    try:
-        request = 'set ' + str(param) + ' ' + str(value) + '\r\n'
-        ser.write(request.encode('utf-8'))
-        response = _parse_response(ser)
-    except Exception as e:
-        print("Error communicating with device")
-        print(e)
-        return False
-
-    try:
-        if(response is not False):
-            if (response.lower() == 'ok'):
-                return True
-            else:
-                print("Bad or unknown response! (" + str(response) + ")")
-                return False
-        else:
+    num_tries = 0
+    while(num_tries < 3):
+        try:
+            request = 'set ' + str(param) + ' ' + str(value) + '\r\n'
+            ser.write(request.encode('utf-8'))
+            response = _parse_response(ser)
+        except Exception as e:
+            print("Error communicating with device")
+            print(e)
             return False
-    except Exception:
-        print("Error parsing response! (" + str(response) + ")")
-        return False
+
+        try:
+            if(response is not False):
+                if (response.lower() == 'ok'):
+                    return True
+                elif(verbose):
+                    print("Command Error, Retrying...")
+            else:
+                return False
+        except Exception:
+            print("Error parsing response! (" + str(response) + ")")
+            return False
+        num_tries = num_tries + 1
+
+    print("Bad or unknown response! (" + str(response) + ")")
+    return False
 
 
 def cmd_get(args):
@@ -238,11 +269,30 @@ def cmd_log(args):
     settings_resp = get_log_settings(settings_to_store)
 
     # Dump out the settings to the log file
-    args.logfile.write("{\"settings\": " + json.dumps(settings_resp))
+    args.logfile.write("{\n  \"settings\": " +
+                       json.dumps(settings_resp, indent=2) + ",\n")
 
     # Start the data portion of the log file
 
-    args.logfile.write(",\"data\": [\n")
+    if(args.compact):
+        args.logfile.write(
+            '\"format\": ['
+            '\"time\",'
+            '\"p_patient\",'
+            '\"f_total\",'
+            '\"o2\",'
+            '\"bpm\",'
+            '\"v_total\",'
+            '\"peep\",'
+            '\"temp\",'
+            '\"bat_pwr\",'
+            '\"bat_charge\",'
+            '\"p_peak\",'
+            '\"v_total_insp\",'
+            '\"v_total_exhl\",'
+            '\"f_peak\"],\n')
+
+    args.logfile.write("\"data\": [\n")
     first = True  # Used to print the ',' between entries
 
     while(1):
@@ -253,32 +303,58 @@ def cmd_log(args):
         if(verbose):
             print(resp)
 
+        if(first):
+            start_time_offset = time.time()
+
         # Ugly, but split up the data and use some sort of name that makes
         # sense the variable names used inside the Arduino code are a bit
         # hard to parse/grok
         data_split = resp.split(',')
-        args.logfile.write(
-            '{13}{{"time": {14}, "last_pressure": {0}, '
-            '"last_flow": {1}, "last_o2": {2}, "last_bpm": {3}, '
-            '"tidal_volume": {4}, "last_peep": {5}, "temperature": {6}, '
-            '"battery_powered": {7}, "current_battery_charge": {8}, '
-            '"current_p_peak": {9}, "current_t_visnp": {10}, '
-            '"current_t_vesp": {11}, "current_vm": {12}}}\n'.format(
-                data_split[0],
-                data_split[1],
-                data_split[2],
-                data_split[3],
-                data_split[4],
-                data_split[5],
-                data_split[6],
-                data_split[7],
-                data_split[8],
-                data_split[9],
-                data_split[10],
-                data_split[11],
-                data_split[12],
-                '' if first else ',',
-                float(time.time())))
+
+        if(args.compact):
+            args.logfile.write(
+                '{13}[{14:.3f},{0},{1},{2},{3},{4},'
+                '{5},{6},{7},{8},{9},{10},{11},{12}]\n'.format(
+                    data_split[0],
+                    data_split[1],
+                    data_split[2],
+                    data_split[3],
+                    data_split[4],
+                    data_split[5],
+                    data_split[6],
+                    data_split[7],
+                    data_split[8],
+                    data_split[9],
+                    data_split[10],
+                    data_split[11],
+                    data_split[12],
+                    ' ' if first else ',',
+                    0.000 if first else float(time.time()-start_time_offset)))
+
+        else:
+            args.logfile.write(
+                '{13}{{"time":{14:.3f},"p_patient":{0},'
+                '"f_total":{1},"o2":{2},"bpm":{3},'
+                '"v_total":{4},"peep":{5},"temp":{6},'
+                '"bat_pwr":{7},"bat_charge":{8},'
+                '"p_peak":{9},"v_total_insp":{10},'
+                '"v_total_exhl":{11},"f_peak":{12}}}\n'.format(
+                    data_split[0],
+                    data_split[1],
+                    data_split[2],
+                    data_split[3],
+                    data_split[4],
+                    data_split[5],
+                    data_split[6],
+                    data_split[7],
+                    data_split[8],
+                    data_split[9],
+                    data_split[10],
+                    data_split[11],
+                    data_split[12],
+                    ' ' if first else ',',
+                    0.000 if first else float(time.time()-start_time_offset)))
+
         if(first):
             first = False
 
@@ -310,10 +386,27 @@ def cmd_console_log(args):
     settings_resp = get_log_settings(settings_to_store)
 
     # Dump out the settings to the log file
-    args.logfile.write("{\"settings\": " + json.dumps(settings_resp))
+    args.logfile.write("{\n  \"settings\": \n" +
+                       json.dumps(settings_resp, indent=2) + ",\n")
+
+    if(args.compact):
+        args.logfile.write(
+            '\"format\": ['
+            '\"time\",'
+            '\"ts\",'
+            '\"flux_inhale\",'
+            '\"p_valve\",'
+            '\"p_patient\",'
+            '\"pv1_ctrl\",'
+            '\"p_slow\",'
+            '\"pv2_ctrl\",'
+            '\"f_vent_raw\",'
+            '\"f_total\",'
+            '\"v_total\",'
+            '\"p_patient_dv2\"],\n')
 
     # Start the data portion of the log file
-    args.logfile.write(",\"data\": [\n")
+    args.logfile.write("\"data\": [\n")
     first = True  # Used to print the ',' between entries
 
     # Start up console logger
@@ -335,26 +428,49 @@ def cmd_console_log(args):
         resp = ser.read_until().decode('utf-8').strip()
         if(verbose):
             print(resp)
+
+        if(first):
+            start_time_offset = time.time()
+
         data_split = resp.split(',')
-        args.logfile.write(
-            '{11}{{"time": {12}, "ts": {0}, '
-            '"last_flow": {1}, "last_pressure0": {2}, "last_pressure1": {3}, '
-            '"pid_monitor": {4}, "pid_monitor2": {5}, "valve2_status": {6}, '
-            '"venturi_flux": {7}, "flow": {8}, '
-            '"tidal_volume": {9}, "dgb_delta": {10}}}\n'.format(
-                data_split[0],
-                data_split[1],
-                data_split[2],
-                data_split[3],
-                data_split[4],
-                data_split[5],
-                data_split[6],
-                data_split[7],
-                data_split[8],
-                data_split[9],
-                data_split[10],
-                '' if first else ',',
-                float(time.time())))
+
+        if(args.compact):
+            args.logfile.write(
+                '{11}[{12:.3f},{0},{1},{2},{3},'
+                '{4},{5},{6},{7},{8},{9},{10}]\n'.format(
+                    data_split[0],
+                    data_split[1],
+                    data_split[2],
+                    data_split[3],
+                    data_split[4],
+                    data_split[5],
+                    data_split[6],
+                    data_split[7],
+                    data_split[8],
+                    data_split[9],
+                    data_split[10],
+                    ' ' if first else ',',
+                    0.000 if first else float(time.time()-start_time_offset)))
+        else:
+            args.logfile.write(
+                '{11}{{"time":{12:.3f},"ts":{0},'
+                '"flux_inhale":{1},"p_valve":{2},"p_patient":{3},'
+                '"pv1_ctrl":{4},"p_slow":{5},"pv2_ctrl":{6},'
+                '"f_vent_raw":{7},"f_total":{8},'
+                '"v_total":{9},"p_patient_dv2":{10}}}'.format(
+                    data_split[0],
+                    data_split[1],
+                    data_split[2],
+                    data_split[3],
+                    data_split[4],
+                    data_split[5],
+                    data_split[6],
+                    data_split[7],
+                    data_split[8],
+                    data_split[9],
+                    data_split[10],
+                    '' if first else ',',
+                    0.000 if first else float(time.time()-start_time_offset)))
 
         if(first):
             first = False
@@ -371,18 +487,19 @@ def cmd_console_log(args):
 
 
 def cmd_load(args):
+    """Command that loads the settings to the device"""
     set_log_settings(args.cfgfile)
 
 
 def cmd_save(args):
+    """Command that saves the settings from the device"""
     settings = get_log_settings(settings_to_store)
-    args.cfgfile.write('{"settings": ')
-    args.cfgfile.write(json.dumps(settings))
-    args.cfgfile.write('}')
+    args.cfgfile.write(json.dumps({"settings": settings}, indent=2))
     args.cfgfile.close()
 
 
 def main():
+    """Main function"""
     global ser
     global verbose
     global rate
@@ -466,6 +583,12 @@ def main():
             'log',
             usage="%(prog)s [option] <file>",
             help="log [option] <file>")
+        parser_log.add_argument(
+            '-c',
+            '--compact',
+            action='store_true',
+            help="Generate compact log"
+        )
         parser_log.add_argument(
             '-r',
             '--rate',
@@ -492,6 +615,12 @@ def main():
             'clog',
             usage="%(prog)s [option] <file>",
             help="clog [option] <file>")
+        parser_clog.add_argument(
+            '-c',
+            '--compact',
+            action='store_true',
+            help="Generate compact log"
+        )
         parser_clog.add_argument(
             '-u',
             '--use-cfg',
